pro dopin,nred=nred,gam=gam,w0=w0,delw=delw,wc1=wc1,wc2=wc2,abs=abs,atm=atm
; cut piece of spectrum out of 'foldspec', convert to velocity scale, and
; subtract continuum.

; nred: reduction factor number of phase bins (only needed if you want to artificially
;  reduce phase resolution)
; gam: gamma velocity to be used (cm/s). Default: saved value
; w0: laboratory wavelength of line used in Angstroms. Default: saved value
; delw: wavelength range around w0 to be used (w0-delw < w < w0+delw). Default: saved value
; wc1, wc2: ranges (two-element arrays: [w0,w1]) of wavelength to be used for continuum definition
;   these are not remembered, since it is assumed they are being varied interactively.
; abs: if set, invert line (for absorption lines)
common dopinsave, gams,w0s,delws,abss

if n_elements(nred) eq 0 then nred=1
cl=2.997e10
;w0=6562.82  ; rest wavelength
if n_elements(gam) eq 0 then begin
  if n_elements(gams) eq 0 then begin
    print,'gamma velocity (cm/s)?'
    read,gam
    gams=gam
  endif else begin
    gam=gams
    print,'gamma used: ',gam
  endelse
endif
gams=gam
if n_elements(w0) eq 0 then begin
  if n_elements(w0s) eq 0 then begin
    print,'lab wavelength of line (Angstrom)?'
    read,w0
    w0s=w0
  endif else begin
    w0=w0s
    print,'lab wavelength used: ',w0
  endelse
endif
w0s=w0
if n_elements(delw) eq 0 then begin
  if n_elements(delws) eq 0 then begin
    print,'wavelength range (Angstrom) around w0 (w0-delw < w < w0+delw)?'
    read,delw
    delws=delw
  endif else begin
    delw=delws
    print,'wavelength interval used: ',delw
  endelse
endif
if n_elements(abs) eq 0 then if n_elements(abss) eq 0 then abs=0 else abs=abss
if n_elements(atm) eq 0 then atm=0
abss=abs
delws=delw

get_lun,lun
openr,lun,'foldspec'
nbin=1
nwo=1
readu,lun,nbin,nwo
trsp=fltarr(nbin,nwo)
pha=fltarr(nbin)
dpha=pha
readu,lun,pha
readu,lun,dpha
w=fltarr(nwo)
wt=fltarr(nbin)
readu,lun,w
if w0+delw gt w(nwo-1) or w0-delw lt w(0) then begin
  print,'w0=',w0,' out of range,',w(0),w(nwo-1)
  stop
endif
readu,lun,trsp
rebin=0
readu,lun,rebin
if rebin then readu,lun,wt

; find sorting of phases
if rebin eq 0 then begin
  phaf=pha mod 1
  indp=where(phaf lt 0.)
  if indp(0) ne -1 then phaf(indp)=phaf(indp)+1
  indp=sort(phaf)
endif

cfi=''
readf,lun,cfi
close,lun
; convert to rest wavelength
nufac=(1+gam/cl)*sqrt(1-(gam/cl)^2)
w=w/nufac
; assuming spectra are F_nu's, apply rel correction
trsp=trsp*nufac

ntab=3
set_plot,'x'
loadct,ntab,/silent
mcol=!d.table_size-1 ; max no of colors of display
xsiz=400
ysiz=150
spac=2
window,2,xsiz=xsiz,ysiz=2*ysiz+spac
ind=indgen(nwo)
iwr=fix(interpol(ind,w,[w0-4*delw,w0+4*delw]))
i0=max([0,iwr(0)])
i1=min([nwo-1,iwr(1)])
ww=w(i0:i1-1)
nww=i1-i0
trw=trsp(*,i0:i1-1)      ; windowed spectrum
trb=total(trw,1)/nbin    ; phase averaged windowed spectrum
ymax=max(trb)
d=mcol*trw/ymax<mcol
nwl=n_elements(trw(0,*))
if rebin eq 0 then for i=0,nwl-1 do d(*,i)=d(indp,i)
tv,congrid(rotate(d,4),xsiz,ysiz),0.,0.

ind=sort(trb)
; pick 25% around the median
f=0.25
indf=ind((1-f)*0.5*nww:(1+f)*0.5*nww)
nwf=n_elements(indf)
clev=total(trw(*,indf),2)/nwf
cla=total(clev)/nbin
trwc=trw
if atm then begin         ; take out atmospheric absorption and phase-average
  for i=0,nww-1 do trw(*,i)=trw(*,i)/clev
  tra=total(trw,1)/nbin
  for i=0,nbin-1 do trw(i,*)=trw(i,*)-tra+1
  for i=0,nww-1 do trw(*,i)=trw(*,i)*clev
endif
for ip=0,nbin-1 do trwc(ip,*)=trw(ip,*)-clev(ip)+cla
d=mcol*trwc/ymax<mcol
if rebin eq 0 then for i=0,nwl-1 do d(*,i)=d(indp,i)
tv,congrid(rotate(d,4),xsiz,ysiz),0.,ysiz+spac

window,0
!p.multi=[0,1,2]
x=ww
trbc=total(trwc,1)/nbin
y=trbc
plot,x,y,xsty=1,xtit='!7k!5(rest)'
ind=indgen(nww)
wr=[w0-delw,w0+delw]
iwr=fix(interpol(ind,ww,wr))+1
i0=max([0,iwr(0)])
i1=min([nwo-1,iwr(1)])
oplot,[ww(i0),ww(i0)],[0,ymax],line=2
oplot,[ww(i1),ww(i1)],[0,ymax],line=2
if n_elements(wc1) ne 2 then begin  ; no proper wavelength range for cont window 1
  print,'click two cursor positions for first continuum window'
  cursor,xc0,yc,/down
  cursor,xc1,yc,/down
  wc1=[xc0,xc1]
  wc1s=wc1
endif
oplot,[wc1(0),wc1(0)],[0,ymax],line=1
oplot,[wc1(1),wc1(1)],[0,ymax],line=1
if n_elements(wc2) ne 2 then begin  ; no proper wavelength range for cont window 2
  print,'click two cursor positions for second continuum window'
  cursor,xc0,yc,/down
  cursor,xc1,yc,/down
  wc2=[xc0,xc1]
  wc2s=wc2
endif
oplot,[wc2(0),wc2(0)],[0,ymax],line=1
oplot,[wc2(1),wc2(1)],[0,ymax],line=1

iwr=fix(interpol(ind,ww,wc1))
ic0=max([0,iwr(0)])
ic1=min([nwo-1,iwr(1)])
iwr=fix(interpol(ind,ww,wc2))
ic2=max([0,iwr(0)])
ic3=min([nwo-1,iwr(1)])
ca1=total(trbc(ic0:ic1-1))/(ic1-ic0)  ; waveln-avg continuum
ca2=total(trbc(ic2:ic3-1))/(ic3-ic2)  ; waveln-avg continuum
;print,'ca1,2=',ca1,ca2
w1=(wc1(0)+wc1(1))/2
w2=(wc2(0)+wc2(1))/2
bl=(ca2-ca1)/(w2-w1)*(x-w1)+ca1
tstr='!7k!5!D0!N='+string(format='(f10.3,''   !7c!5='',e10.2)',w0,gam)
v=((x/w0)^2-1)*cl/(1+(x/w0)^2)
if abs then y=bl-y else y=y-bl
plot,v,y,xsty=1,xtit='v(cm/s)',title=tstr
oplot,[v(0),v(nww-1)],[0,0]
oplot,[0,0],[0,ymax]
vmi=((ww(i0)/w0)^2-1)/(1+(ww(i0)/w0)^2)*cl
vma=((ww(i1-1)/w0)^2-1)/(1+(ww(i1-1)/w0)^2)*cl
oplot,[vmi,vmi],[0,ymax],line=2
oplot,[vma,vma],[0,ymax],line=2

nw=i1-i0
nbr=nbin/nred
ind=nred*indgen(nbr)
dopsp=fltarr(nbr,nw)
for i=0,nbr-1 do dopsp(i,*)=trwc(ind(i),i0:i1-1)
vdop=((ww(i0:i1-1)/w0)^2-1)*cl/(1+(ww(i0:i1-1)/w0)^2)

get_lun,lun
openw,lun,'dopin'
v1=((w1/w0)^2-1)*cl/(1+(w1/w0)^2)
v2=((w2/w0)^2-1)*cl/(1+(w2/w0)^2)
; construct linearly varying continuum 
bl=(ca2-ca1)/(v2-v1)*(vdop-v1)+ca1
; subtract from spectrum
for i=0,nbr-1 do dopsp(i,*)=dopsp(i,*)-bl
printf,lun,nbr,nw,w0
printf,lun,gam,abs,atm,'    '+cfi
printf,lun,pha
printf,lun,1
printf,lun,dpha
printf,lun,vdop
if abs ne 0 then dopsp=-dopsp
printf,lun,dopsp
close,lun
free_lun,lun
;stop
!p.multi=[0,1,1]
return
end

